/* 实现功能
 1: 蒙版
 2：判断界面弹出与否
 3：动画时间
 4：自定义界面
 5：设置初始值
 6: 传值  block
 */

#import "ZKPopView.h"

#define Time_Animation 0.25                                         // 动画时间
#define Screen_Width   [UIScreen mainScreen].bounds.size.width      // 屏幕宽度
#define Screen_Height  [UIScreen mainScreen].bounds.size.height     // 屏幕高度
#define Bottom_Height  280                                          // 下界面高度
#define Bar_Height     60                                           // 工具条
@interface ZKPopView ()<UIPickerViewDelegate, UIPickerViewDataSource>
{
    BOOL     _isPop;                                // 是否弹出界面
    UIView  *_bottomView;                           // 自定义弹出界面下界面
    UIView  *_mengbanView;                          // 蒙版视图
    UILabel *_titleLb;                              // 工具条标题
    UIPickerView *_timeDatePicker;                  // 底部-主界面
}
@end

@implementation ZKPopView
// 初始化方法
- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self)
    {
        /*
         // 这种方法不好，他的子类都收到透明度的影响。
         self.backgroundColor = [UIColor grayColor];
         self.alpha = 0.3;
         */
        
        //1： 采用这种方式
        self.backgroundColor = [UIColor clearColor];
        self.frame = CGRectMake(0, 0, [UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height);

        //2： 添加一层蒙版
        [self addMengbanView];
        
        //3：创建底部界面
        [self createBottomView];
        
        // 4:显示底部视图
        _isPop = NO;
        [self showPopView];
    }
    return self;
}

#pragma mark - 创建蒙版
// 添加蒙版
- (UIView *)addMengbanView
{
    if (!_mengbanView)
    {
        UIView *mengbanView = [UIView new];
        mengbanView.frame = CGRectMake(0, Screen_Height, Screen_Width, Screen_Height);
        mengbanView.backgroundColor = [UIColor grayColor];
        mengbanView.alpha = 0.3f;
        [self addSubview:mengbanView];
        _mengbanView = mengbanView;
    }
    return _mengbanView;
}

// 添加底部界面
- (UIView *)createBottomView
{
    if (!_bottomView)
    {
        UIView *bottomView = [UIView new];
        bottomView.backgroundColor = [UIColor whiteColor];
        bottomView.frame = CGRectMake(0, Screen_Height + Screen_Height - Bottom_Height - Bar_Height, Screen_Width, Bottom_Height + Bar_Height);
        [self addSubview:bottomView];
        _bottomView = bottomView;
        [self createCustomBottomView:bottomView];
    }
    return _bottomView;
}

// 添加工具条
- (void)createTopBar:(UIView *)bottomView
{
    UIView *line = [UIView new];
    line.frame = CGRectMake(0, Bar_Height - 1, Screen_Width, 1);
    line.backgroundColor = [UIColor lightGrayColor];
    [bottomView addSubview:line];
    
    CGFloat  buttonWidth = Screen_Width/3;
    // 标题
    UILabel *tittle = [UILabel new];
    tittle.font = [UIFont systemFontOfSize:18];
    tittle.textAlignment = NSTextAlignmentCenter;
    tittle.textColor  = [UIColor blackColor];
    tittle.text = @"选择时间";
    tittle.frame = CGRectMake(buttonWidth, 0, buttonWidth, Bar_Height);
    [bottomView addSubview:tittle];
    _titleLb = tittle;
    
    // 左按钮取消
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, buttonWidth, Bar_Height);
    [button setTitle:@"取消" forState:UIControlStateNormal];
    [button setTitle:@"取消" forState:UIControlStateHighlighted];
    button.titleLabel.font = [UIFont systemFontOfSize:18];
    [button addTarget:self action:@selector(cancelButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    [button setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    [button setTitleColor:[UIColor blackColor] forState:UIControlStateHighlighted];
    [bottomView addSubview:button];
    
    // 有按钮确定
    UIButton *confirmBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    confirmBtn.frame = CGRectMake(Screen_Width - buttonWidth, 0, buttonWidth, Bar_Height);
    [confirmBtn setTitle:@"确定" forState:UIControlStateNormal];
    [confirmBtn setTitle:@"确定" forState:UIControlStateHighlighted];
    [confirmBtn addTarget:self action:@selector(conformButtonClick:) forControlEvents:UIControlEventTouchUpInside];
    confirmBtn.titleLabel.font = [UIFont systemFontOfSize:18];
    [confirmBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
    [confirmBtn setTitleColor:[UIColor grayColor] forState:UIControlStateHighlighted];
    [bottomView addSubview:confirmBtn];
}

// 设置弹出框标题
- (void)setTitle:(NSString *)title
{
    if (_title != title)
    {
        _title = title;
        _titleLb.text = title;
    }
}

// 设置默认的标题
- (void)setSelectTittle:(NSString *)selectTittle
{
    if (_selectTittle != selectTittle)
    {
        _selectTittle = selectTittle;
        
        // 设置默认的标题
        [self setDefaultTittle];
    }
}

// 设置数组
- (void)setPopArray:(NSArray *)popArray
{
    if (_popArray != popArray)
    {
        _popArray = popArray;
        
        // 刷新表
        [_timeDatePicker reloadComponent:0];
        
        // 设置默认的标题
        [self setDefaultTittle];
    }
}

#pragma mark - 点击事件
// 取消按钮
- (void)cancelButtonClick:(UIButton *)button
{
    [self hidePopView];
}

// 确定按钮
- (void)conformButtonClick:(UIButton *)button
{
    [self hidePopView];
    
    [self selectTittleOfPicker];
}

// 点击屏幕
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self hidePopView];
    
    [self selectTittleOfPicker];
}

#pragma mark - 其他
// 显示界面
- (void)showPopView
{
    [UIView animateWithDuration:Time_Animation animations:^{
        if (_isPop == NO)
        {
            UIWindow *window = [UIApplication sharedApplication].keyWindow;
            [window addSubview:self];
            
            _mengbanView.frame = CGRectMake(0, 0, Screen_Width, Screen_Height);
            _bottomView.frame = CGRectMake(0,  Screen_Height - Bottom_Height - Bar_Height, Screen_Width, Bottom_Height + Bar_Height);
        }
    }completion:^(BOOL finished) {
        if (_isPop == NO) {
            _isPop = YES;
        }
    }];
}

// 隐藏界面
- (void)hidePopView
{
   [UIView animateWithDuration:Time_Animation animations:^{
       if (_isPop == YES)
       {
           _mengbanView.frame = CGRectMake(0, Screen_Height, Screen_Width, Screen_Height);
           _bottomView.frame = CGRectMake(0,  Screen_Height + Screen_Height - Bottom_Height - Bar_Height, Screen_Width, Bottom_Height + Bar_Height);
//           NSLog(@"33333");
       }
    }completion:^(BOOL finished) {
        if (_isPop == YES)
        {
            _isPop = NO;
            [self removeFromSuperview];
//            NSLog(@"4444");
        }
    }];
}

/** 选中标题 */
- (void)selectTittleOfPicker
{
    NSInteger row = [_timeDatePicker selectedRowInComponent:0];
    NSString *selectedTittle = [self.popArray objectAtIndex:row];
    if (self.passValue)
    {
        // 3：block实现
        self.passValue(selectedTittle);
    }
}

// 初始化默认的标题
- (void)setDefaultTittle
{
    // 默认选择
    if(_popArray && _selectTittle)
    {
        NSInteger index = [_popArray indexOfObject:_selectTittle];
        if ( index  >= 0 &&  index < _popArray.count)
        {
            [_timeDatePicker selectRow:index inComponent:0 animated:YES];
        }
    }

}
#pragma mark - 具体界面
/********************************************************/
- (void)createCustomBottomView:(UIView *)bottomView
{
    // 工具条
    [self createTopBar:bottomView];
    
    [self createPickerView:bottomView];
}

- (UIPickerView *)createPickerView:(UIView *)bottomView
{
    if (!_timeDatePicker)
    {
        UIPickerView *pickerView =[[UIPickerView alloc] init];
        pickerView.frame = CGRectMake(0, Bar_Height, Screen_Width, Bottom_Height);
        pickerView.dataSource = self;
        pickerView.delegate = self;
        pickerView.backgroundColor = [UIColor whiteColor];
        [bottomView  addSubview:pickerView];
        _timeDatePicker = pickerView;
    }
    return _timeDatePicker;
}

#pragma mark - UIPickerViewDataSource, UIPickerViewDelegate

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return _popArray.count;
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    NSString *name = _popArray[row];
    return name;
}

//- (UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(UIView *)view
//{
//    float cw = [self pickerView:pickerView widthForComponent:component];
//    float ch = [self pickerView:pickerView rowHeightForComponent:component];
//
//    UILabel* cell = view ? (UILabel *) view : [[UILabel alloc] initWithFrame:CGRectMake(0, 0, cw, ch)];
//    cell.font = [UIFont systemFontOfSize:18];
//    cell.text = [self pickerView:pickerView titleForRow:row forComponent:component];
//    cell.textAlignment = NSTextAlignmentCenter;
//    cell.backgroundColor = [UIColor clearColor];
//
//    return cell;
//}
@end
